`timescale 1ns/1ps

module hycube(
    clk,
    reset,
    chip_en,
    data_in,
    address_in,
    data_out,
    scan_data_or_addr,
    read_write,
    data_out_valid,
    data_addr_valid,
    scan_start_exec,
    bist_en,
    exec_end
);

input logic                             reset;
input logic                             clk;
input logic                             chip_en;

input  logic                            scan_start_exec;

output  logic                           data_out_valid;
input  [1:0]                            data_addr_valid;

input [31:0]                            data_in;
input [19:0]                            address_in;
output [31:0]                           data_out;
input  logic                            scan_data_or_addr;
input  logic                            read_write;
output logic                            exec_end;
input logic                             bist_en;

import TopPkg::*;
import SMARTPkg::*;

parameter DATA_WIDTH 			= 32;
parameter CM_WIDTH			= 64;
parameter DATA_DEPTH			= 9;

parameter CM_DEPTH			= 32;
parameter CM_DEPTH_BITS                 = $clog2(CM_DEPTH);


parameter DM_DEPTH			= 512;
parameter DM_DEPTH_BITS                 = $clog2(DM_DEPTH);

parameter NUM_RESET_SYNC_STAGES         = 2;

localparam TILE_NUM_INPUT_PORTS         = 6;
localparam TILE_NUM_OUTPUT_PORTS        = 7;
localparam NUM_CLUSTERS			= 4;

reg start_exec;
reg [31:0] data_out;
reg [63:0] look_up_table_reg;
reg [CM_WIDTH-1:0] cm_data;
reg [DATA_WIDTH-1:0] dm_data;
reg [DATA_WIDTH-1:0] dm_bit_en;
reg [CM_WIDTH-1:0] cm_bit_en;
reg bist_success;
reg [1:0] data_addr_valid_reg;
reg data_or_addr_reg;
wire chip_en_dm0;
wire chip_en_dm8;
wire chip_en_dm16;
wire chip_en_dm24;
wire chip_en_dm32;
wire chip_en_dm40;
wire chip_en_dm48;
wire chip_en_dm56;

always @ (posedge clk)
begin
if (reset)
begin
	data_addr_valid_reg <=2'b00;
	data_or_addr_reg <= 1'b0;
end
else if (chip_en) begin
	data_addr_valid_reg <=data_addr_valid;
	data_or_addr_reg <=scan_data_or_addr;
end 
end 

/*
reg [11:0] dm_en;

always @ (posedge clk)
begin
if (reset) begin
	dm_en = 16'b0000000000000000;
end
else begin
        if(scan_data_or_addr==1'b1 || address_in[18]==1'b0 || data_addr_valid== 2'b00 || address_in[19] == 1'b1)
                dm_en = 16'b0000000000000000;
        else begin //this is if it is a DM
                case(address_in[17:14])
                4'b0000: dm_en = 16'b0000000000000001;
                4'b0001: dm_en = 16'b0000000000000010;
                4'b0010: dm_en = 16'b0000000000000100;
                4'b0011: dm_en = 16'b0000000000001000;
                4'b0100: dm_en = 16'b0000000000010000;
                4'b0101: dm_en = 16'b0000000000100000;
                4'b0110: dm_en = 16'b0000000001000000;
                4'b0111: dm_en = 16'b0000000010000000;
                4'b1000: dm_en = 16'b0000000100000000;
                4'b1001: dm_en = 16'b0000001000000000;
                4'b1010: dm_en = 16'b0000010000000000;
                4'b1011: dm_en = 16'b0000100000000000;
		default: dm_en = 16'b0000000000000000;
                endcase 
        end 
end
end

assign chip_en_dm0 = (dm_en == 16'b0000000000000001) ? 1'b0 : 1'b1;
assign chip_en_dm8 = (dm_en == 16'b0000000000000010) ? 1'b0 : 1'b1;
assign chip_en_dm16 = (dm_en == 16'b0000000000000100) ? 1'b0 : 1'b1;
assign chip_en_dm24 = (dm_en == 16'b0000000000001000) ? 1'b0 : 1'b1;
assign chip_en_dm32 = (dm_en == 16'b0000000000010000) ? 1'b0 : 1'b1;
assign chip_en_dm40 = (dm_en == 16'b0000000000100000) ? 1'b0 : 1'b1;
assign chip_en_dm48 = (dm_en == 16'b0000000001000000) ? 1'b0 : 1'b1;
assign chip_en_dm56 = (dm_en == 16'b0000000010000000) ? 1'b0 : 1'b1;
assign chip_en_dm7 = (dm_en == 16'b0000000100000000) ? 1'b0 : 1'b1;
assign chip_en_dm15 = (dm_en == 16'b0000001000000000) ? 1'b0 : 1'b1;
assign chip_en_dm23 = (dm_en == 16'b0000010000000000) ? 1'b0 : 1'b1;
assign chip_en_dm31 = (dm_en == 16'b0000100000000000) ? 1'b0 : 1'b1;

always @ (posedge clk)
begin
if (reset) begin
	dm_data = data_in[DATA_WIDTH-1:0];
	dm_bit_en = {DATA_WIDTH{1'b0}};
end
else begin
if (address_in[18]==1'b1 && address_in[19]==1'b0) begin
        if(address_in[0]==1'b0) begin
                if (data_addr_valid==2'b01) begin
			dm_data = {{DATA_WIDTH/2{1'b0}},data_in [DATA_WIDTH/2-1:0]};
			dm_bit_en = {DATA_WIDTH{1'b1}};
		end
                else if (data_addr_valid==2'b10) begin
			dm_data = {data_in [DATA_WIDTH-1:DATA_WIDTH/2],{DATA_WIDTH/2{1'b0}}};
			dm_bit_en = {DATA_WIDTH{1'b1}};
                end
                else if (data_addr_valid==2'b11) begin
			dm_data = data_in [DATA_WIDTH-1:0];
			dm_bit_en = {DATA_WIDTH{1'b1}};
                end
                else if (data_addr_valid==2'b00) begin
			dm_data = data_in [DATA_WIDTH-1:0];
  			dm_bit_en = {DATA_WIDTH{1'b0}};
                end
        end 
        else if(address_in[0]==1'b1) begin
                if (data_addr_valid==2'b01) begin
			dm_data = {data_in [DATA_WIDTH/2-1:0],{DATA_WIDTH/2{1'b0}}};
			dm_bit_en = {DATA_WIDTH{1'b1}};
                end
                else if (data_addr_valid==2'b10) begin
			dm_data = {{DATA_WIDTH/2{1'b0}},data_in [15:8]};
			dm_bit_en = {DATA_WIDTH{1'b1}};
                end
                else if (data_addr_valid==2'b11) begin
			dm_data = data_in [DATA_WIDTH-1:0];
			dm_bit_en = {DATA_WIDTH{1'b1}};
                end
                else if (data_addr_valid==2'b00) begin
			dm_data = data_in [DATA_WIDTH-1:0];
			dm_bit_en = {DATA_WIDTH{1'b0}};
                end
        end
	else begin                        
			dm_data = data_in [DATA_WIDTH-1:0];
			dm_bit_en = {DATA_WIDTH{1'b0}};
	end	
	end 
	else begin                        
		dm_data = data_in [DATA_WIDTH-1:0];
		dm_bit_en = {DATA_WIDTH{1'b0}};
	end	
end
end
 */
//always @ (posedge clk)
always_comb
begin
if (reset) begin
	cm_data = {{CM_WIDTH{1'b0}}};
	cm_bit_en = {CM_WIDTH{1'b1}};
end
else begin
if (address_in[18]==1'b0 && address_in[19]==1'b0) begin
        if(address_in[2:1]==2'b00) begin
                if (data_addr_valid==2'b01) begin
                        cm_data = {{56{1'b0}},data_in[7:0]};
			cm_bit_en = {{56{1'b0}},{8{1'b1}}};
                end
                else if (data_addr_valid==2'b10) begin
                        cm_data = {{48{1'b0}},data_in[15:8],{8{1'b0}}};
			cm_bit_en = {{48{1'b0}},{8{1'b1}},{8{1'b0}}};
                end
                else if (data_addr_valid==2'b11) begin
                        cm_data = {{48{1'b0}},data_in[15:0]};
			cm_bit_en = {{48{1'b0}},{16{1'b1}}};
                end
                else if (data_addr_valid==2'b00) begin
                        cm_data = {{48{1'b0}},data_in[15:0]};
			cm_bit_en = {{48{1'b0}},{8{1'b0}},{8{1'b0}}};
                end
        end
        else if(address_in[2:1]==2'b01) begin
                if (data_addr_valid==2'b01) begin
                        cm_data = {{40{1'b0}},data_in[7:0],{16{1'b0}}};
			cm_bit_en = {{40{1'b0}},{8{1'b1}},{16{1'b0}}};
                end
                else if (data_addr_valid==2'b10) begin
                        cm_data = {{32{1'b0}},data_in[15:8],{8{1'b0}},{16{1'b0}}};
			cm_bit_en = {{32{1'b0}},{8{1'b1}},{24{1'b0}}};
                end
                else if (data_addr_valid==2'b11) begin
                        cm_data = {{32{1'b0}},data_in[15:0],{16{1'b0}}};
			cm_bit_en = {{32{1'b0}},{16{1'b1}},{16{1'b0}}};
                end
                else if (data_addr_valid==2'b00) begin
                        cm_data = {{32{1'b0}},data_in[15:0],{16{1'b0}}};
			cm_bit_en = {{32{1'b0}},{16{1'b0}},{16{1'b0}}};
                end
        end
        else if(address_in[2:1]==2'b10) begin
                if (data_addr_valid==2'b01) begin
                        cm_data = {{24{1'b0}},data_in[7:0],{32{1'b0}}};
			cm_bit_en = {{24{1'b0}},{8{1'b1}},{32{1'b0}}};
                end
                else if (data_addr_valid==2'b10) begin
                        cm_data = {{16{1'b0}},data_in[15:8],{8{1'b0}},{32{1'b0}}};
			cm_bit_en = {{16{1'b0}},{8{1'b1}},{40{1'b0}}};
                end
                else if (data_addr_valid==2'b11) begin
                        cm_data = {{16{1'b0}},data_in[15:0],{32{1'b0}}};
			cm_bit_en = {{16{1'b0}},{16{1'b1}},{32{1'b0}}};
                end
                else if (data_addr_valid==2'b00) begin
                        cm_data = {{16{1'b0}},data_in[15:0],{32{1'b0}}};
			cm_bit_en = {{16{1'b0}},{16{1'b0}},{32{1'b0}}};
                end
        end
        else if(address_in[2:1]==2'b11) begin 
                if (data_addr_valid==2'b01) begin
                        cm_data = {{8{1'b0}},data_in[7:0],{48{1'b0}}};
			cm_bit_en = {{8{1'b0}},{8{1'b1}},{48{1'b0}}}; 
                end
                else if (data_addr_valid==2'b10) begin
                        cm_data = {data_in[15:8],{8{1'b0}},{48{1'b0}}};
			cm_bit_en = {{8{1'b1}},{8{1'b0}},{48{1'b1}}}; 
                end
                else if (data_addr_valid==2'b11) begin
                        cm_data = {data_in[15:0],{48{1'b0}}};
			cm_bit_en = {{8{1'b1}},{8{1'b1}},{48{1'b0}}}; 
                end
                else if (data_addr_valid==2'b00) begin
                        cm_data = {data_in[15:0],{48{1'b0}}};
			cm_bit_en = {{8{1'b0}},{8{1'b0}},{48{1'b0}}}; 
                end
        end
	else begin
		cm_data = {{CM_WIDTH{1'b0}}};
		cm_bit_en = {{CM_WIDTH{1'b0}}};
	end
end 
else begin
	
	cm_data = {{CM_WIDTH{1'b0}}};
	cm_bit_en = {{CM_WIDTH{1'b0}}};
end
end
end 

reg [35:0] cm_en;
reg [CM_WIDTH-1:0] cm_bit_en0;
reg [CM_WIDTH-1:0] cm_bit_en1;
reg [CM_WIDTH-1:0] cm_bit_en2;
reg [CM_WIDTH-1:0] cm_bit_en3;
reg [CM_WIDTH-1:0] cm_bit_en4;
reg [CM_WIDTH-1:0] cm_bit_en5;
reg [CM_WIDTH-1:0] cm_bit_en6;
reg [CM_WIDTH-1:0] cm_bit_en7;
reg [CM_WIDTH-1:0] cm_bit_en8;
reg [CM_WIDTH-1:0] cm_bit_en9;
reg [CM_WIDTH-1:0] cm_bit_en10;
reg [CM_WIDTH-1:0] cm_bit_en11;
reg [CM_WIDTH-1:0] cm_bit_en12;
reg [CM_WIDTH-1:0] cm_bit_en13;
reg [CM_WIDTH-1:0] cm_bit_en14;
reg [CM_WIDTH-1:0] cm_bit_en15;
reg [CM_WIDTH-1:0] cm_bit_en16;
reg [CM_WIDTH-1:0] cm_bit_en17;
reg [CM_WIDTH-1:0] cm_bit_en18;
reg [CM_WIDTH-1:0] cm_bit_en19;
reg [CM_WIDTH-1:0] cm_bit_en20;
reg [CM_WIDTH-1:0] cm_bit_en21;
reg [CM_WIDTH-1:0] cm_bit_en22;
reg [CM_WIDTH-1:0] cm_bit_en23;
reg [CM_WIDTH-1:0] cm_bit_en24;
reg [CM_WIDTH-1:0] cm_bit_en25;
reg [CM_WIDTH-1:0] cm_bit_en26;
reg [CM_WIDTH-1:0] cm_bit_en27;
reg [CM_WIDTH-1:0] cm_bit_en28;
reg [CM_WIDTH-1:0] cm_bit_en29;
reg [CM_WIDTH-1:0] cm_bit_en30;
reg [CM_WIDTH-1:0] cm_bit_en31;
reg [CM_WIDTH-1:0] cm_bit_en32;
reg [CM_WIDTH-1:0] cm_bit_en33;
reg [CM_WIDTH-1:0] cm_bit_en34;
reg [CM_WIDTH-1:0] cm_bit_en35;

//always @ (posedge clk)
always_comb
begin
if (reset) begin
	cm_en = 36'b000000000000000000000000000000000000;
end
else begin
        if(data_addr_valid==2'b00 || scan_data_or_addr==1'b1 || (address_in[18]==1'b1) || (address_in[19]==1'b1)) begin
                cm_en = 36'b000000000000000000000000000000000000;
	end
        else begin
                case(address_in[12:7])
                6'b000000: cm_en = 36'b000000000000000000000000000000000001;
                6'b000001: cm_en = 36'b000000000000000000000000000000000010;
                6'b000010: cm_en = 36'b000000000000000000000000000000000100;
                6'b000011: cm_en = 36'b000000000000000000000000000000001000;
                6'b000100: cm_en = 36'b000000000000000000000000000000010000;
                6'b000101: cm_en = 36'b000000000000000000000000000000100000;
                6'b000110: cm_en = 36'b000000000000000000000000000001000000;
                6'b000111: cm_en = 36'b000000000000000000000000000010000000;
                6'b001000: cm_en = 36'b000000000000000000000000000100000000;
                6'b001001: cm_en = 36'b000000000000000000000000001000000000;
                6'b001010: cm_en = 36'b000000000000000000000000010000000000;
                6'b001011: cm_en = 36'b000000000000000000000000100000000000;
                6'b001100: cm_en = 36'b000000000000000000000001000000000000;
                6'b001101: cm_en = 36'b000000000000000000000010000000000000;
                6'b001110: cm_en = 36'b000000000000000000000100000000000000;
                6'b001111: cm_en = 36'b000000000000000000001000000000000000;
                6'b010000: cm_en = 36'b000000000000000000010000000000000000;
                6'b010001: cm_en = 36'b000000000000000000100000000000000000;
                6'b010010: cm_en = 36'b000000000000000001000000000000000000;
                6'b010011: cm_en = 36'b000000000000000010000000000000000000;
                6'b010100: cm_en = 36'b000000000000000100000000000000000000;
                6'b010101: cm_en = 36'b000000000000001000000000000000000000;
                6'b010110: cm_en = 36'b000000000000010000000000000000000000;
                6'b010111: cm_en = 36'b000000000000100000000000000000000000;
                6'b011000: cm_en = 36'b000000000001000000000000000000000000;
                6'b011001: cm_en = 36'b000000000010000000000000000000000000;
                6'b011010: cm_en = 36'b000000000100000000000000000000000000;
                6'b011011: cm_en = 36'b000000001000000000000000000000000000;
                6'b011100: cm_en = 36'b000000010000000000000000000000000000;
                6'b011101: cm_en = 36'b000000100000000000000000000000000000;
                6'b011110: cm_en = 36'b000001000000000000000000000000000000;
                6'b011111: cm_en = 36'b000010000000000000000000000000000000;
                6'b100000: cm_en = 36'b000100000000000000000000000000000000;
                6'b100001: cm_en = 36'b001000000000000000000000000000000000;
                6'b100010: cm_en = 36'b010000000000000000000000000000000000;
                6'b100011: cm_en = 36'b100000000000000000000000000000000000;
		default  : cm_en = 36'b000000000000000000000000000000000000;
                endcase 
        end
end
end

assign cm_bit_en0  = (cm_en == 36'b000000000000000000000000000000000001) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en1  = (cm_en == 36'b000000000000000000000000000000000010) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en2  = (cm_en == 36'b000000000000000000000000000000000100) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en3  = (cm_en == 36'b000000000000000000000000000000001000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en4  = (cm_en == 36'b000000000000000000000000000000010000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en5  = (cm_en == 36'b000000000000000000000000000000100000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en6  = (cm_en == 36'b000000000000000000000000000001000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en7  = (cm_en == 36'b000000000000000000000000000010000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en8  = (cm_en == 36'b000000000000000000000000000100000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en9  = (cm_en == 36'b000000000000000000000000001000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en10 = (cm_en == 36'b000000000000000000000000010000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en11 = (cm_en == 36'b000000000000000000000000100000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en12 = (cm_en == 36'b000000000000000000000001000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en13 = (cm_en == 36'b000000000000000000000010000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en14 = (cm_en == 36'b000000000000000000000100000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en15 = (cm_en == 36'b000000000000000000001000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en16 = (cm_en == 36'b000000000000000000010000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en17 = (cm_en == 36'b000000000000000000100000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en18 = (cm_en == 36'b000000000000000001000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en19 = (cm_en == 36'b000000000000000010000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en20 = (cm_en == 36'b000000000000000100000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en21 = (cm_en == 36'b000000000000001000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en22 = (cm_en == 36'b000000000000010000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en23 = (cm_en == 36'b000000000000100000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en24 = (cm_en == 36'b000000000001000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en25 = (cm_en == 36'b000000000010000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en26 = (cm_en == 36'b000000000100000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en27 = (cm_en == 36'b000000001000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en28 = (cm_en == 36'b000000010000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en29 = (cm_en == 36'b000000100000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en30 = (cm_en == 36'b000001000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en31 = (cm_en == 36'b000010000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en32 = (cm_en == 36'b000100000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en33 = (cm_en == 36'b001000000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en34 = (cm_en == 36'b010000000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
assign cm_bit_en35 = (cm_en == 36'b100000000000000000000000000000000000) ? cm_bit_en : {64{1'b0}};
/*
always @ (posedge clk)
begin
if (reset)
begin
        look_up_table_reg <= {64{1'b0}};
end
else if (chip_en)
begin
        if (address_in[19] == 1'b1) begin
                if(address_in[1:0]==2'b00) look_up_table_reg[15:0] <= data_in;
                if(address_in[1:0]==2'b01) look_up_table_reg[31:16] <= data_in;
                if(address_in[1:0]==2'b10) look_up_table_reg[47:32] <= data_in;
                if(address_in[1:0]==2'b11) look_up_table_reg[51:48] <= data_in;
        end
end
end
*/
always @ (posedge clk)
begin
if (reset)
begin
        start_exec <= 1'b0;
end
else if (chip_en)
begin
                start_exec <= scan_start_exec;
end
end

logic [NUM_RESET_SYNC_STAGES-1:0]       r__reset__pff;

FlitFixed                w__flit_in                          [NUM_TILES_Y-1:0][NUM_TILES_X-1:0][TILE_NUM_INPUT_PORTS-1-2:0];
FlitFixed                w__flit_out                         [NUM_TILES_Y-1:0][NUM_TILES_X-1:0][TILE_NUM_OUTPUT_PORTS-1-3:0];
logic scan_chain_out_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];

logic [CM_WIDTH-1:0] data_out_cm_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic rd_en_shifted_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic [CM_DEPTH_BITS-1:0] addr_shifted_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic [DM_DEPTH_BITS-1:0] addr_dm_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic [DATA_WIDTH-1:0] bit_en_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic [DATA_WIDTH-1:0] data_in_dm_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic rd_en_dm_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
logic wr_en_dm_out [NUM_TILES_Y-1:0][NUM_TILES_X-1:0];
wire [3:0] addr;
/*
wire [DM_DEPTH_BITS-1:0] addr_dm_shifted0;
wire [DATA_WIDTH-1:0] bit_en_shifted0;
wire [DATA_WIDTH-1:0] data_in_dm_shifted0;
wire chip_en_dm_shifted0;
wire wr_en_dm_shifted0;
wire [DATA_WIDTH-1:0] data_out_dm0;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted8;
wire [DATA_WIDTH-1:0] bit_en_shifted8;
wire [DATA_WIDTH-1:0] data_in_dm_shifted8;
wire chip_en_dm_shifted8;
wire wr_en_dm_shifted8;
wire [DATA_WIDTH-1:0] data_out_dm8;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted16;
wire [DATA_WIDTH-1:0] bit_en_shifted16;
wire [DATA_WIDTH-1:0] data_in_dm_shifted16;
wire chip_en_dm_shifted16;
wire wr_en_dm_shifted16;
wire [DATA_WIDTH-1:0] data_out_dm16;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted24;
wire [DATA_WIDTH-1:0] bit_en_shifted24;
wire [DATA_WIDTH-1:0] data_in_dm_shifted24;
wire chip_en_dm_shifted24;
wire wr_en_dm_shifted24;
wire [DATA_WIDTH-1:0] data_out_dm24;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted32;
wire [DATA_WIDTH-1:0] bit_en_shifted32;
wire [DATA_WIDTH-1:0] data_in_dm_shifted32;
wire chip_en_dm_shifted32;
wire wr_en_dm_shifted32;
wire [DATA_WIDTH-1:0] data_out_dm32;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted40;
wire [DATA_WIDTH-1:0] bit_en_shifted40;
wire [DATA_WIDTH-1:0] data_in_dm_shifted40;
wire chip_en_dm_shifted40;
wire wr_en_dm_shifted40;
wire [DATA_WIDTH-1:0] data_out_dm40;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted48;
wire [DATA_WIDTH-1:0] bit_en_shifted48;
wire [DATA_WIDTH-1:0] data_in_dm_shifted48;
wire chip_en_dm_shifted48;
wire wr_en_dm_shifted48;
wire [DATA_WIDTH-1:0] data_out_dm48;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted56;
wire [DATA_WIDTH-1:0] bit_en_shifted56;
wire [DATA_WIDTH-1:0] data_in_dm_shifted56;
wire chip_en_dm_shifted56;
wire wr_en_dm_shifted56;
wire [DATA_WIDTH-1:0] data_out_dm56;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted7;
wire [DATA_WIDTH-1:0] bit_en_shifted7;
wire [DATA_WIDTH-1:0] data_in_dm_shifted7;
wire chip_en_dm_shifted7;
wire wr_en_dm_shifted7;
wire [DATA_WIDTH-1:0] data_out_dm7;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted15;
wire [DATA_WIDTH-1:0] bit_en_shifted15;
wire [DATA_WIDTH-1:0] data_in_dm_shifted15;
wire chip_en_dm_shifted15;
wire wr_en_dm_shifted15;
wire [DATA_WIDTH-1:0] data_out_dm15;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted23;
wire [DATA_WIDTH-1:0] bit_en_shifted23;
wire [DATA_WIDTH-1:0] data_in_dm_shifted23;
wire chip_en_dm_shifted23;
wire wr_en_dm_shifted23;
wire [DATA_WIDTH-1:0] data_out_dm23;

wire [DM_DEPTH_BITS-1:0] addr_dm_shifted31;
wire [DATA_WIDTH-1:0] bit_en_shifted31;
wire [DATA_WIDTH-1:0] data_in_dm_shifted31;
wire chip_en_dm_shifted31;
wire wr_en_dm_shifted31;
wire [DATA_WIDTH-1:0] data_out_dm31;

wire #(1) clk_shifted_A;
wire #(1) clk_shifted_B;
wire clk_shifted;

/*reg [DM_DEPTH_BITS-1:0] addr0_bist;
reg [DATA_WIDTH-1:0] data_in0_bist;
reg [DATA_WIDTH-1:0] bit_en0_bist;
reg chip_en0_bist;
reg wr_en0_bist;

reg [DM_DEPTH_BITS-1:0] addr8_bist;
reg [DATA_WIDTH-1:0] data_in8_bist;
reg [DATA_WIDTH-1:0] bit_en8_bist;
reg chip_en8_bist;
reg wr_en8_bist;

reg [DM_DEPTH_BITS-1:0] addr16_bist;
reg [DATA_WIDTH-1:0] data_in16_bist;
reg [DATA_WIDTH-1:0] bit_en16_bist;
reg chip_en16_bist;
reg wr_en16_bist;

reg [DM_DEPTH_BITS-1:0] addr24_bist;
reg [DATA_WIDTH-1:0] data_in24_bist;
reg [DATA_WIDTH-1:0] bit_en24_bist;
reg chip_en24_bist;
reg wr_en24_bist;

reg [DM_DEPTH_BITS-1:0] addr32_bist;
reg [DATA_WIDTH-1:0] data_in32_bist;
reg [DATA_WIDTH-1:0] bit_en32_bist;
reg chip_en32_bist;
reg wr_en32_bist;

reg [DM_DEPTH_BITS-1:0] addr40_bist;
reg [DATA_WIDTH-1:0] data_in40_bist;
reg [DATA_WIDTH-1:0] bit_en40_bist;
reg chip_en40_bist;
reg wr_en40_bist;

reg [DM_DEPTH_BITS-1:0] addr48_bist;
reg [DATA_WIDTH-1:0] data_in48_bist;
reg [DATA_WIDTH-1:0] bit_en48_bist;
reg chip_en48_bist;
reg wr_en48_bist;

reg [DM_DEPTH_BITS-1:0] addr56_bist;
reg [DATA_WIDTH-1:0] data_in56_bist;
reg [DATA_WIDTH-1:0] bit_en56_bist;
reg chip_en56_bist;
reg wr_en56_bist;

reg [DM_DEPTH_BITS-1:0] addr7_bist;
reg [DATA_WIDTH-1:0] data_in7_bist;
reg [DATA_WIDTH-1:0] bit_en7_bist;
reg chip_en7_bist;
reg wr_en7_bist;

reg [DM_DEPTH_BITS-1:0] addr15_bist;
reg [DATA_WIDTH-1:0] data_in15_bist;
reg [DATA_WIDTH-1:0] bit_en15_bist;
reg chip_en15_bist;
reg wr_en15_bist;

reg [DM_DEPTH_BITS-1:0] addr23_bist;
reg [DATA_WIDTH-1:0] data_in23_bist;
reg [DATA_WIDTH-1:0] bit_en23_bist;
reg chip_en23_bist;
reg wr_en23_bist;

reg [DM_DEPTH_BITS-1:0] addr31_bist;
reg [DATA_WIDTH-1:0] data_in31_bist;
reg [DATA_WIDTH-1:0] bit_en31_bist;
reg chip_en31_bist;
reg wr_en31_bist;

reg [1:0] count_dm_bist;

always @(posedge clk)
begin
if (reset) begin
	addr0_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in0_bist <= {DATA_WIDTH{1'b1}};
	bit_en0_bist <= {DATA_WIDTH{1'b1}};
	chip_en0_bist <= 1'b1;
	wr_en0_bist <= 1'b1;
	
	addr8_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in8_bist <= {DATA_WIDTH{1'b1}};
	bit_en8_bist <= {DATA_WIDTH{1'b1}};
	chip_en8_bist <= 1'b1;
	wr_en8_bist <= 1'b1;

	addr16_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in16_bist <= {DATA_WIDTH{1'b1}};
	bit_en16_bist <= {DATA_WIDTH{1'b1}};
	chip_en16_bist <= 1'b1;
	wr_en16_bist <= 1'b1;

	addr24_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in24_bist <= {DATA_WIDTH{1'b1}};
	bit_en24_bist <= {DATA_WIDTH{1'b1}};
	chip_en24_bist <= 1'b1;
	wr_en24_bist <= 1'b1;

	addr32_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in32_bist <= {DATA_WIDTH{1'b1}};
	bit_en32_bist <= {DATA_WIDTH{1'b1}};
	chip_en32_bist <= 1'b1;
	wr_en32_bist <= 1'b1;

	addr40_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in40_bist <= {DATA_WIDTH{1'b1}};
	bit_en40_bist <= {DATA_WIDTH{1'b1}};
	chip_en40_bist <= 1'b1;
	wr_en40_bist <= 1'b1;

	addr48_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in48_bist <= {DATA_WIDTH{1'b1}};
	bit_en48_bist <= {DATA_WIDTH{1'b1}};
	chip_en48_bist <= 1'b1;
	wr_en48_bist <= 1'b1;

	addr56_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in56_bist <= {DATA_WIDTH{1'b1}};
	bit_en56_bist <= {DATA_WIDTH{1'b1}};
	chip_en56_bist <= 1'b1;
	wr_en56_bist <= 1'b1;

	addr7_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in7_bist <= {DATA_WIDTH{1'b1}};
	bit_en7_bist <= {DATA_WIDTH{1'b1}};
	chip_en7_bist <= 1'b1;
	wr_en7_bist <= 1'b1;

	addr15_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in15_bist <= {DATA_WIDTH{1'b1}};
	bit_en15_bist <= {DATA_WIDTH{1'b1}};
	chip_en15_bist <= 1'b1;
	wr_en15_bist <= 1'b1;

	addr23_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in23_bist <= {DATA_WIDTH{1'b1}};
	bit_en23_bist <= {DATA_WIDTH{1'b1}};
	chip_en23_bist <= 1'b1;
	wr_en23_bist <= 1'b1;

	addr31_bist <= {DM_DEPTH_BITS{1'b1}};
	data_in31_bist <= {DATA_WIDTH{1'b1}};
	bit_en31_bist <= {DATA_WIDTH{1'b1}};
	chip_en31_bist <= 1'b1;
	wr_en31_bist <= 1'b0;
	count_dm_bist <= 2'b00;	
end
else if (chip_en&&bist_en) begin
	if (count_dm_bist==0) begin
        	addr0_bist <= 14'b000011111;
		data_in0_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
		bit_en0_bist <= {DATA_WIDTH{1'b0}};
	        chip_en0_bist <= 1'b0;
	        wr_en0_bist <= 1'b0;
	        
		addr8_bist <= 14'b000011111;
	        data_in8_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en8_bist <= {DATA_WIDTH{1'b0}};
	        chip_en8_bist <= 1'b1;
	        wr_en8_bist <= 1'b0;
	        
		addr16_bist <= 14'b000011111;
	        data_in16_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en16_bist <= {DATA_WIDTH{1'b0}};
	        chip_en16_bist <= 1'b0;
	        wr_en16_bist <= 1'b0;
	        
		addr24_bist <= 14'b000011111;
	        data_in24_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
        	bit_en24_bist <= {DATA_WIDTH{1'b0}};
	        chip_en24_bist <= 1'b1;
        	wr_en24_bist <= 1'b0;
        	
		addr32_bist <= 14'b000011111;
		data_in32_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
		bit_en32_bist <= {DATA_WIDTH{1'b0}};
	        chip_en32_bist <= 1'b0;
	        wr_en32_bist <= 1'b0;
	        
		addr40_bist <= 14'b000011111;
	        data_in40_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en40_bist <= {DATA_WIDTH{1'b0}};
	        chip_en40_bist <= 1'b1;
	        wr_en40_bist <= 1'b0;
	        
		addr48_bist <= 14'b000011111;
	        data_in48_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en48_bist <= {DATA_WIDTH{1'b0}};
	        chip_en48_bist <= 1'b0;
	        wr_en48_bist <= 1'b0;
	        
		addr56_bist <= 14'b000011111;
	        data_in56_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
        	bit_en56_bist <= {DATA_WIDTH{1'b0}};
	        chip_en56_bist <= 1'b1;
        	wr_en56_bist <= 1'b0;

        	addr7_bist <= 14'b000011111;
		data_in7_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
		bit_en7_bist <= {DATA_WIDTH{1'b0}};
	        chip_en7_bist <= 1'b0;
	        wr_en7_bist <= 1'b0;
	        
		addr15_bist <= 14'b000011111;
	        data_in15_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en15_bist <= {DATA_WIDTH{1'b0}};
	        chip_en15_bist <= 1'b1;
	        wr_en15_bist <= 1'b0;
	        
		addr23_bist <= 14'b000011111;
	        data_in23_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en23_bist <= {DATA_WIDTH{1'b0}};
	        chip_en23_bist <= 1'b0;
	        wr_en23_bist <= 1'b0;
	        
		addr31_bist <= 14'b000011111;
	        data_in31_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
        	bit_en31_bist <= {DATA_WIDTH{1'b0}};
	        chip_en31_bist <= 1'b1;
        	wr_en31_bist <= 1'b0;
	end
	if (count_dm_bist==1) begin
        	addr0_bist <= 14'b110011111;
	        data_in0_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en0_bist <= {DATA_WIDTH{1'b0}};
	        chip_en0_bist <= 1'b1;
	        wr_en0_bist <= 1'b0;
	        
		addr8_bist <= 14'b110011111;
	        data_in8_bist <= {{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}}};
	        bit_en8_bist <= {DATA_WIDTH{1'b0}};
	        chip_en8_bist <= 1'b0;
	        wr_en8_bist <= 1'b0;
	        
		addr16_bist <= 14'b110011111;
	        data_in16_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en16_bist <= {DATA_WIDTH{1'b0}};
	        chip_en16_bist <= 1'b1;
	        wr_en16_bist <= 1'b0;
	        
		addr24_bist <= 14'b110011111;
	        data_in24_bist <= {{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}}};
        	bit_en24_bist <= {DATA_WIDTH{1'b0}};
	        chip_en24_bist <= 1'b0;
        	wr_en24_bist <= 1'b0;
        	
		
        	addr32_bist <= 14'b110011111;
	        data_in32_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en32_bist <= {DATA_WIDTH{1'b0}};
	        chip_en32_bist <= 1'b1;
	        wr_en32_bist <= 1'b0;
	        
		addr40_bist <= 14'b110011111;
	        data_in40_bist <= {{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}}};
	        bit_en40_bist <= {DATA_WIDTH{1'b0}};
	        chip_en40_bist <= 1'b0;
	        wr_en40_bist <= 1'b0;
	        
		addr48_bist <= 14'b110011111;
	        data_in48_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en48_bist <= {DATA_WIDTH{1'b0}};
	        chip_en48_bist <= 1'b1;
	        wr_en48_bist <= 1'b0;
	        
		addr56_bist <= 14'b110011111;
	        data_in56_bist <= {{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}}};
        	bit_en56_bist <= {DATA_WIDTH{1'b0}};
	        chip_en56_bist <= 1'b0;
        	wr_en56_bist <= 1'b0;
        	
		addr7_bist <= 14'b110011111;
	        data_in7_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en7_bist <= {DATA_WIDTH{1'b0}};
	        chip_en7_bist <= 1'b1;
	        wr_en7_bist <= 1'b0;
	        
		addr15_bist <= 14'b110011111;
	        data_in15_bist <= {{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}}};
	        bit_en15_bist <= {DATA_WIDTH{1'b0}};
	        chip_en15_bist <= 1'b0;
	        wr_en15_bist <= 1'b0;
	        
		addr23_bist <= 14'b110011111;
	        data_in23_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en23_bist <= {DATA_WIDTH{1'b0}};
	        chip_en23_bist <= 1'b1;
	        wr_en23_bist <= 1'b0;
	        
		addr31_bist <= 14'b110011111;
	        data_in31_bist <= {{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}},{DATA_WIDTH/4{1'b0}},{DATA_WIDTH/4{1'b1}}};
        	bit_en31_bist <= {DATA_WIDTH{1'b0}};
	        chip_en31_bist <= 1'b0;
        	wr_en31_bist <= 1'b0;
	end
	if (count_dm_bist==2) begin
        	addr0_bist <= 14'b110011111;
	        data_in0_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en0_bist <= {DATA_WIDTH{1'b0}};
	        chip_en0_bist <= 1'b0;
	        wr_en0_bist <= 1'b1;
	        
		addr8_bist <= 14'b110011111;
	        data_in8_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en8_bist <= {DATA_WIDTH{1'b0}};
	        chip_en8_bist <= 1'b1;
	        wr_en8_bist <= 1'b1;
	        
		addr16_bist <= 14'b110011111;
	        data_in16_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en16_bist <= {DATA_WIDTH{1'b0}};
	        chip_en16_bist <= 1'b0;
	        wr_en16_bist <= 1'b1;
	        
		addr24_bist <= 14'b110011111;
	        data_in24_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
        	bit_en24_bist <= {DATA_WIDTH{1'b0}};
	        chip_en24_bist <= 1'b1;
        	wr_en24_bist <= 1'b1;
        	
		addr32_bist <= 14'b110011111;
	        data_in32_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en32_bist <= {DATA_WIDTH{1'b0}};
	        chip_en32_bist <= 1'b0;
	        wr_en32_bist <= 1'b1;
	        
		addr40_bist <= 14'b110011111;
	        data_in40_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en40_bist <= {DATA_WIDTH{1'b0}};
	        chip_en40_bist <= 1'b1;
	        wr_en40_bist <= 1'b1;
	        
		addr48_bist <= 14'b110011111;
	        data_in48_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en48_bist <= {DATA_WIDTH{1'b0}};
	        chip_en48_bist <= 1'b0;
	        wr_en48_bist <= 1'b1;
	        
		addr56_bist <= 14'b110011111;
	        data_in56_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
        	bit_en56_bist <= {DATA_WIDTH{1'b0}};
	        chip_en56_bist <= 1'b1;
        	wr_en56_bist <= 1'b1;
        	
		addr7_bist <= 14'b110011111;
	        data_in7_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en7_bist <= {DATA_WIDTH{1'b0}};
	        chip_en7_bist <= 1'b0;
	        wr_en7_bist <= 1'b1;
	        
		addr15_bist <= 14'b110011111;
	        data_in15_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en15_bist <= {DATA_WIDTH{1'b0}};
	        chip_en15_bist <= 1'b1;
	        wr_en15_bist <= 1'b1;
	        
		addr23_bist <= 14'b110011111;
	        data_in23_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en23_bist <= {DATA_WIDTH{1'b0}};
	        chip_en23_bist <= 1'b0;
	        wr_en23_bist <= 1'b1;
	        
		addr31_bist <= 14'b110011111;
	        data_in31_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
        	bit_en31_bist <= {DATA_WIDTH{1'b0}};
	        chip_en31_bist <= 1'b1;
        	wr_en31_bist <= 1'b1;
	end
	if (count_dm_bist==3) begin
        	addr0_bist <= 14'b000011111;
	        data_in0_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en0_bist <= {DATA_WIDTH{1'b0}};
	        chip_en0_bist <= 1'b1;
	        wr_en0_bist <= 1'b1;
	        
		addr8_bist <= 14'b000011111;
	        data_in8_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en8_bist <= {DATA_WIDTH{1'b0}};
	        chip_en8_bist <= 1'b0;
	        wr_en8_bist <= 1'b1;
	        
		addr16_bist <= 14'b000011111;
	        data_in16_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en16_bist <= {DATA_WIDTH{1'b0}};
	        chip_en16_bist <= 1'b1;
	        wr_en16_bist <= 1'b1;
	        
		addr24_bist <= 14'b000011111;
	        data_in24_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
		bit_en24_bist <= {DATA_WIDTH{1'b0}};
	        chip_en24_bist <= 1'b0;
        	wr_en24_bist <= 1'b1;
        	
		addr32_bist <= 14'b000011111;
	        data_in32_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en32_bist <= {DATA_WIDTH{1'b0}};
	        chip_en32_bist <= 1'b1;
	        wr_en32_bist <= 1'b1;
	        
		addr40_bist <= 14'b000011111;
	        data_in40_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en40_bist <= {DATA_WIDTH{1'b0}};
	        chip_en40_bist <= 1'b0;
	        wr_en40_bist <= 1'b1;
	        
		addr48_bist <= 14'b000011111;
	        data_in48_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en48_bist <= {DATA_WIDTH{1'b0}};
	        chip_en48_bist <= 1'b1;
	        wr_en48_bist <= 1'b1;
	        
		addr56_bist <= 14'b000011111;
	        data_in56_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
		bit_en56_bist <= {DATA_WIDTH{1'b0}};
	        chip_en56_bist <= 1'b0;
        	wr_en56_bist <= 1'b1;
        
		addr7_bist <= 14'b000011111;
	        data_in7_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en7_bist <= {DATA_WIDTH{1'b0}};
	        chip_en7_bist <= 1'b1;
	        wr_en7_bist <= 1'b1;
	        
		addr15_bist <= 14'b000011111;
	        data_in15_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
	        bit_en15_bist <= {DATA_WIDTH{1'b0}};
	        chip_en15_bist <= 1'b0;
	        wr_en15_bist <= 1'b1;
	        
		addr23_bist <= 14'b000011111;
	        data_in23_bist <= {{DATA_WIDTH/2{1'b0}},{DATA_WIDTH/2{1'b1}}};
	        bit_en23_bist <= {DATA_WIDTH{1'b0}};
	        chip_en23_bist <= 1'b1;
	        wr_en23_bist <= 1'b1;
	        
		addr31_bist <= 14'b000011111;
	        data_in31_bist <= {{DATA_WIDTH/2{1'b1}},{DATA_WIDTH/2{1'b0}}};
		bit_en31_bist <= {DATA_WIDTH{1'b0}};
	        chip_en31_bist <= 1'b0;
        	wr_en31_bist <= 1'b1;
        	
	end
	count_dm_bist <= count_dm_bist + 1'b1;
end
end

always @(posedge clk) begin
if (reset) begin
	bist_success <= 1'b0;
end
else if (chip_en && bist_en) begin
	if (data_out_dm0==16'b11001100 && data_out_dm8==16'b00001111 && data_out_dm16==16'b00110011 && data_out_dm24==16'b11110000 && data_out_dm32==16'b11001100 && data_out_dm40==16'b00001111 && data_out_dm48==16'b00110011 && data_out_dm56==16'b11110000 && data_out_dm7==16'b11001100 && data_out_dm15==16'b00001111 && data_out_dm23==16'b00110011 && data_out_dm31==16'b11110000)
		bist_success <= 1'b1;
	else
		bist_success <= 1'b0;
end
else
	bist_success <= 1'b0;
end

IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem0 (            //1K each
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted0),
.CEN_B(chip_en_dm_shifted8),
.RDWEN_A(wr_en_dm_shifted0),
.RDWEN_B(wr_en_dm_shifted8),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted0[8]),
.AW_A(addr_dm_shifted0[4:0]),
.AC_A(addr_dm_shifted0[7:5]),
.AS_B(addr_dm_shifted8[8]),
.AW_B(addr_dm_shifted8[4:0]),
.AC_B(addr_dm_shifted8[7:5]),
.D_A(data_in_dm_shifted0),
.D_B(data_in_dm_shifted8),
.BW_A(bit_en_shifted0),
.BW_B(bit_en_shifted8),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en0_bist),
.T_CEN_B(chip_en8_bist),
.T_RDWEN_A(wr_en0_bist),
.T_RDWEN_B(wr_en8_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr0_bist[8]),
.T_AS_B(addr8_bist[8]),
.T_AW_A(addr0_bist[4:0]),
.T_AW_B(addr8_bist[4:0]),
.T_AC_A(addr0_bist[7:5]),
.T_AC_B(addr8_bist[7:5]),
.T_D_A(data_in0_bist),
.T_D_B(data_in8_bist),
.T_BW_A(bit_en0_bist),
.T_BW_B(bit_en8_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm0),
.Q_B(data_out_dm8),
.OBSV_CTL_A(),
.OBSV_CTL_B());


IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem1 (            //2K each
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted16),
.CEN_B(chip_en_dm_shifted24),
.RDWEN_A(wr_en_dm_shifted16),
.RDWEN_B(wr_en_dm_shifted24),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted16[8]),
.AW_A(addr_dm_shifted16[4:0]),
.AC_A(addr_dm_shifted16[7:5]),
.AS_B(addr_dm_shifted24[8]),
.AW_B(addr_dm_shifted24[4:0]),
.AC_B(addr_dm_shifted24[7:5]),
.D_A(data_in_dm_shifted16),
.D_B(data_in_dm_shifted24),
.BW_A(bit_en_shifted16),
.BW_B(bit_en_shifted24),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en16_bist),
.T_CEN_B(chip_en24_bist),
.T_RDWEN_A(wr_en16_bist),
.T_RDWEN_B(wr_en24_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr16_bist[8]),
.T_AS_B(addr24_bist[8]),
.T_AW_A(addr16_bist[4:0]),
.T_AW_B(addr24_bist[4:0]),
.T_AC_A(addr16_bist[7:5]),
.T_AC_B(addr24_bist[7:5]),
.T_D_A(data_in16_bist),
.T_D_B(data_in24_bist),
.T_BW_A(bit_en16_bist),
.T_BW_B(bit_en24_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm16),
.Q_B(data_out_dm24),
.OBSV_CTL_A(),
.OBSV_CTL_B());


IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem2 (            
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted32),
.CEN_B(chip_en_dm_shifted40),
.RDWEN_A(wr_en_dm_shifted32),
.RDWEN_B(wr_en_dm_shifted40),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted32[8]),
.AW_A(addr_dm_shifted32[4:0]),
.AC_A(addr_dm_shifted32[7:5]),
.AS_B(addr_dm_shifted40[8]),
.AW_B(addr_dm_shifted40[4:0]),
.AC_B(addr_dm_shifted40[7:5]),
.D_A(data_in_dm_shifted32),
.D_B(data_in_dm_shifted40),
.BW_A(bit_en_shifted32),
.BW_B(bit_en_shifted40),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en32_bist),
.T_CEN_B(chip_en40_bist),
.T_RDWEN_A(wr_en32_bist),
.T_RDWEN_B(wr_en40_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr32_bist[8]),
.T_AS_B(addr40_bist[8]),
.T_AW_A(addr32_bist[4:0]),
.T_AW_B(addr40_bist[4:0]),
.T_AC_A(addr32_bist[7:5]),
.T_AC_B(addr40_bist[7:5]),
.T_D_A(data_in32_bist),
.T_D_B(data_in40_bist),
.T_BW_A(bit_en32_bist),
.T_BW_B(bit_en40_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm32),
.Q_B(data_out_dm40),
.OBSV_CTL_A(),
.OBSV_CTL_B());

IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem3 (            
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted48),
.CEN_B(chip_en_dm_shifted56),
.RDWEN_A(wr_en_dm_shifted48),
.RDWEN_B(wr_en_dm_shifted56),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted48[8]),
.AW_A(addr_dm_shifted48[4:0]),
.AC_A(addr_dm_shifted48[7:5]),
.AS_B(addr_dm_shifted56[8]),
.AW_B(addr_dm_shifted56[4:0]),
.AC_B(addr_dm_shifted56[7:5]),
.D_A(data_in_dm_shifted48),
.D_B(data_in_dm_shifted56),
.BW_A(bit_en_shifted48),
.BW_B(bit_en_shifted56),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en48_bist),
.T_CEN_B(chip_en56_bist),
.T_RDWEN_A(wr_en48_bist),
.T_RDWEN_B(wr_en56_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr48_bist[8]),
.T_AS_B(addr56_bist[8]),
.T_AW_A(addr48_bist[4:0]),
.T_AW_B(addr56_bist[4:0]),
.T_AC_A(addr48_bist[7:5]),
.T_AC_B(addr56_bist[7:5]),
.T_D_A(data_in48_bist),
.T_D_B(data_in56_bist),
.T_BW_A(bit_en48_bist),
.T_BW_B(bit_en56_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm48),
.Q_B(data_out_dm56),
.OBSV_CTL_A(),
.OBSV_CTL_B());

IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem4 (            //1K each
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted7),
.CEN_B(chip_en_dm_shifted15),
.RDWEN_A(wr_en_dm_shifted7),
.RDWEN_B(wr_en_dm_shifted15),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted7[8]),
.AW_A(addr_dm_shifted7[4:0]),
.AC_A(addr_dm_shifted7[7:5]),
.AS_B(addr_dm_shifted15[8]),
.AW_B(addr_dm_shifted15[4:0]),
.AC_B(addr_dm_shifted15[7:5]),
.D_A(data_in_dm_shifted7),
.D_B(data_in_dm_shifted15),
.BW_A(bit_en_shifted7),
.BW_B(bit_en_shifted15),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en7_bist),
.T_CEN_B(chip_en15_bist),
.T_RDWEN_A(wr_en7_bist),
.T_RDWEN_B(wr_en15_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr7_bist[8]),
.T_AS_B(addr15_bist[8]),
.T_AW_A(addr7_bist[4:0]),
.T_AW_B(addr15_bist[4:0]),
.T_AC_A(addr7_bist[7:5]),
.T_AC_B(addr15_bist[7:5]),
.T_D_A(data_in7_bist),
.T_D_B(data_in15_bist),
.T_BW_A(bit_en7_bist),
.T_BW_B(bit_en15_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm7),
.Q_B(data_out_dm15),
.OBSV_CTL_A(),
.OBSV_CTL_B());

IN22FDX_SDPV_NFVG_W00512B032M08C064 data_mem5 (            //1K each
.CLK_A(clk_shifted),
.CLK_B(clk_shifted),
.CEN_A(chip_en_dm_shifted23),
.CEN_B(chip_en_dm_shifted31),
.RDWEN_A(wr_en_dm_shifted23),
.RDWEN_B(wr_en_dm_shifted31),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AS_A(addr_dm_shifted23[8]),
.AW_A(addr_dm_shifted23[4:0]),
.AC_A(addr_dm_shifted23[7:5]),
.AS_B(addr_dm_shifted31[8]),
.AW_B(addr_dm_shifted31[4:0]),
.AC_B(addr_dm_shifted31[7:5]),
.D_A(data_in_dm_shifted23),
.D_B(data_in_dm_shifted31),
.BW_A(bit_en_shifted23),
.BW_B(bit_en_shifted31),
.T_BIST(bist_en),
.T_LOGIC(1'b0),
.T_CEN_A(chip_en23_bist),
.T_CEN_B(chip_en31_bist),
.T_RDWEN_A(wr_en23_bist),
.T_RDWEN_B(wr_en31_bist),
.T_DEEPSLEEP(1'b0),
.T_POWERGATE(1'b0),
.T_AS_A(addr23_bist[8]),
.T_AS_B(addr31_bist[8]),
.T_AW_A(addr23_bist[4:0]),
.T_AW_B(addr31_bist[4:0]),
.T_AC_A(addr23_bist[7:5]),
.T_AC_B(addr31_bist[7:5]),
.T_D_A(data_in23_bist),
.T_D_B(data_in31_bist),
.T_BW_A(bit_en23_bist),
.T_BW_B(bit_en31_bist),
.T_WBT(1'b0),
.T_STAB(1'b0),
.MA_SAWL(2'b01),
.MA_WL(2'b00),
.MA_WRAS(2'b00),
.MA_WRASD(1'b0),
.Q_A(data_out_dm23),
.Q_B(data_out_dm31),
.OBSV_CTL_A(),
.OBSV_CTL_B());

*/
/*
always @(posedge clk)                                                                              
if (reset)
	exec_end <= 1'b1;
else if (scan_start_exec==1'b1 && start_exec==1'b0)                                                
        exec_end <= 1'b0;                                                                          
else if (chip_en&&((addr_dm_shifted0=={DM_DEPTH_BITS{1'b1}} || addr_dm_shifted8=={DM_DEPTH_BITS{1'b1}} || addr_dm_shifted16=={DM_DEPTH_BITS{1'b1}} || addr_dm_shifted24=={DM_DEPTH_BITS{1'b1}}) && (data_in_dm_shifted0[DATA_WIDTH-1:DATA_WIDTH-8] == {{7{1'b0}},1'b1} || data_in_dm_shifted8[DATA_WIDTH-1:DATA_WIDTH-8] == {{7{1'b0}},1'b1} || data_in_dm_shifted16[DATA_WIDTH-1:DATA_WIDTH-8] == {{7{1'b0}},1'b1} || data_in_dm_shifted24[DATA_WIDTH-1:DATA_WIDTH-8] == {{7{1'b0}},1'b1})))
	exec_end <= 1'b1;        

*/
//assign exec_end = ((addr_dm_shifted0=={DM_DEPTH_BITS{1'b1}}) && (data_in_dm_shifted0=={{DATA_WIDTH-1{1'b0}}, 1'b1}));
//wire [DATA_WIDTH-1:0] dm_data_out;
//assign dm_data_out = (chip_en_dm_shifted0 == 1'b0) ? data_out_dm0[DATA_WIDTH-1:0] : (chip_en_dm_shifted8 == 1'b0) ? data_out_dm8[DATA_WIDTH-1:0] :(chip_en_dm_shifted16 == 1'b0) ? data_out_dm16[DATA_WIDTH-1:0] :(chip_en_dm_shifted24 == 1'b0) ? data_out_dm24[DATA_WIDTH-1:0] : (chip_en_dm_shifted32 == 1'b0) ? data_out_dm32[DATA_WIDTH-1:0] : (chip_en_dm_shifted40 == 1'b0) ? data_out_dm40[DATA_WIDTH-1:0] :(chip_en_dm_shifted48 == 1'b0) ? data_out_dm48[DATA_WIDTH-1:0] :(chip_en_dm_shifted56 == 1'b0) ? data_out_dm56[DATA_WIDTH-1:0] :(chip_en_dm_shifted7 == 1'b0) ? data_out_dm7[DATA_WIDTH-1:0] : (chip_en_dm_shifted15 == 1'b0) ? data_out_dm15[DATA_WIDTH-1:0] :(chip_en_dm_shifted23 == 1'b0) ? data_out_dm23[DATA_WIDTH-1:0] :(chip_en_dm_shifted31 == 1'b0) ? data_out_dm31[DATA_WIDTH-1:0] :{DATA_WIDTH{1'b0}};


//------------------------------------------------------------------------------

generate
genvar i, j, k;

for(i = 0; i < NUM_TILES_Y; i++)
begin: y
    for(j = 0; j < NUM_TILES_X; j++)
    begin: x
        TileXYId                        my_xy_id;

        FlitFixed        my_flit_in                      [TILE_NUM_INPUT_PORTS-1-2:0];
        FlitFixed        my_flit_out                     [TILE_NUM_OUTPUT_PORTS-1-3:0];


wire [CM_DEPTH_BITS-1:0] loop_start;
wire [CM_DEPTH_BITS-1:0] loop_end;
wire [CM_WIDTH-1:0] data_in;
wire wr_en;

wire [4:0] addr;
wire [DM_DEPTH_BITS-1:0] addr_dm;
wire [DATA_WIDTH-1:0] bit_en;
wire [DATA_WIDTH-1:0] data_in_dm;
wire rd_en_dm;
wire wr_en_dm;
wire is_dm_tile;

reg wr_en_shifted;
reg rd_en_shifted;

wire scan_chain_out;
        //----------------------------------------------------------------------
        // Flit, Credit, SSR
        //----------------------------------------------------------------------
        assign my_flit_in[EAST]     = w__flit_in[i][j][EAST];
        assign my_flit_in[SOUTH]    = w__flit_in[i][j][SOUTH];
        assign my_flit_in[WEST]     = w__flit_in[i][j][WEST];
        assign my_flit_in[NORTH]    = w__flit_in[i][j][NORTH];

        assign w__flit_out[i][j][EAST]      = my_flit_out[EAST];
        assign w__flit_out[i][j][SOUTH]     = my_flit_out[SOUTH];
        assign w__flit_out[i][j][WEST]      = my_flit_out[WEST];
        assign w__flit_out[i][j][NORTH]     = my_flit_out[NORTH];

        //----------------------------------------------------------------------

        `ifdef GATE_SIM
        //----------------------------------------------------------------------
        // Gate level connection
        //----------------------------------------------------------------------
        assign my_gate_flit_in                      = my_flit_in;
        assign `LINK_DELAY my_flit_out           = my_gate_flit_out;

        //----------------------------------------------------------------------
        `endif

wire [DATA_WIDTH-1:0] data_out_dm;
//modify to specify data into tile by wangbo
//assign data_out_dm = (my_xy_id==6'b000000) ? data_out_dm0[DATA_WIDTH-1:0] : (my_xy_id==6'b001000) ? data_out_dm8[DATA_WIDTH-1:0] : (my_xy_id==6'b010000) ? data_out_dm16[DATA_WIDTH-1:0]  : (my_xy_id==6'b011000) ? data_out_dm24[DATA_WIDTH-1:0] : (my_xy_id==6'b100000) ? data_out_dm32[DATA_WIDTH-1:0] : (my_xy_id==6'b101000) ? data_out_dm40[DATA_WIDTH-1:0] : (my_xy_id==6'b110000) ? data_out_dm48[DATA_WIDTH-1:0] : (my_xy_id==6'b111000) ? data_out_dm56[DATA_WIDTH-1:0] : (my_xy_id==6'b000111) ? data_out_dm7[DATA_WIDTH-1:0] : (my_xy_id==6'b001111) ? data_out_dm15[DATA_WIDTH-1:0] : (my_xy_id==6'b010111) ? data_out_dm23[DATA_WIDTH-1:0] : (my_xy_id==6'b011111) ? data_out_dm31[DATA_WIDTH-1:0] : (my_xy_id==6'b100111) ? data_out_dm39[DATA_WIDTH-1:0] : (my_xy_id==6'b101111) ? data_out_dm47[DATA_WIDTH-1:0] : (my_xy_id==6'b110111) ? data_out_dm55[DATA_WIDTH-1:0] : (my_xy_id==6'b111111) ? data_out_dm63[DATA_WIDTH-1:0] : {DATA_WIDTH{1'b0}} ;
//assign data_out_dm = (i==0 && j==0) ? data_out_dm0[DATA_WIDTH-1:0] : (i==0 && j==1) ? data_out_dm8[DATA_WIDTH-1:0] : (i==0 && j==2) ? data_out_dm16[DATA_WIDTH-1:0]  : (i==0 && j==3) ? data_out_dm24[DATA_WIDTH-1:0] : (i==0 && j==4) ? data_out_dm32[DATA_WIDTH-1:0] : (i==0 && j==5) ? data_out_dm40[DATA_WIDTH-1:0] : (i==5 && j==0) ? data_out_dm48[DATA_WIDTH-1:0] : (i==5 && j==1) ? data_out_dm56[DATA_WIDTH-1:0]  : (i==5 && j==2) ? data_out_dm7[DATA_WIDTH-1:0] : (i==5 && j==3) ? data_out_dm15[DATA_WIDTH-1:0] : (i==5 && j==4) ? data_out_dm23[DATA_WIDTH-1:0] : (i==5 && j==5) ? data_out_dm31[DATA_WIDTH-1:0] :{DATA_WIDTH{1'b0}} ;


//assign is_dm_tile = (my_xy_id==6'b000000) ? 1'b1 : (my_xy_id==6'b001000) ? 1'b1 : (my_xy_id==6'b010000) ? 1'b1 : (my_xy_id==6'b011000) ? 1'b1 : (my_xy_id==6'b100000) ? 1'b1 : (my_xy_id==6'b101000) ? 1'b1 : (my_xy_id==6'b110000) ? 1'b1 : (my_xy_id==6'b111000) ? 1'b1 : (my_xy_id==6'b000111) ? 1'b1 : (my_xy_id==6'b001111) ? 1'b1 : (my_xy_id==6'b010111) ? 1'b1 : (my_xy_id==6'b011111) ? 1'b1 : (my_xy_id==6'b100111) ? 1'b1 : (my_xy_id==6'b101111) ? 1'b1 : (my_xy_id==6'b1110111) ? 1'b1 : (my_xy_id==6'b111111) ? 1'b1 :1'b0 ;
assign is_dm_tile = (i==0 && j==0) ? 1'b1 : (i==0 && j==1) ? 1'b1 : (i==0 && j==2) ? 1'b1 : (i==0 && j==3) ? 1'b1 : (i==0 && j==4) ? 1'b1 : (i==0 && j==5) ? 1'b1 : (i==5 && j==0) ? 1'b1 : (i==5 && j==1) ? 1'b1 : (i==5 && j==2) ? 1'b1 : (i==5 && j==3) ? 1'b1: (i==5 && j==4) ? 1'b1 : (i==5 && j==5) ? 1'b1 : 1'b0 ;

wire [CM_WIDTH-1:0] data_out;
reg [CM_DEPTH_BITS-1:0] addr_shifted;
reg [CM_WIDTH-1:0] control_reg_data;

always @(posedge clk) begin
if (reset) begin
	wr_en_shifted <= 1'b1;
	rd_en_shifted <= 1'b1;
end
else if(chip_en) begin
	if (scan_start_exec) begin
		rd_en_shifted <= 1'b0;
		wr_en_shifted <= 1'b1;
	end
	else begin
		rd_en_shifted <= ~read_write;
		wr_en_shifted <= read_write;
	end
end
end
//THILINI:check
wire [3:0] loop_end_dash;

assign loop_end_dash = (loop_end < loop_start) ? (loop_end + 4'b1000) : loop_end; 


//wangbo
assign addr_dm_out[i][j] = addr_dm;
assign bit_en_out[i][j] = bit_en;
assign data_in_dm_out[i][j] = data_in_dm;
assign rd_en_dm_out[i][j] = rd_en_dm;
assign wr_en_dm_out[i][j] = wr_en_dm;
assign rd_en_shifted_out[i][j] = rd_en_shifted;
assign data_out_cm_out[i][j] = data_out;

reg [4:0] jumpl_reg;
always @(posedge clk)
begin
if (reset)
	jumpl_reg <= 5'b00000;
else if(chip_en) begin
	jumpl_reg <= control_reg_data[34:30];
end
end

always @(posedge clk)
begin
if (reset)
	addr_shifted <= 4'b0000;
else if (chip_en) begin
	if (start_exec) begin
		if (control_reg_data[34:30]==5'b11110 && jumpl_reg != 5'b11110) begin
			addr_shifted <= {1'b0,control_reg_data[45:43]};
		end
		else if ((addr_shifted >= loop_end_dash) || (addr_shifted < loop_start)) begin
			addr_shifted <= loop_start;
		end
		else begin
			addr_shifted <= addr_shifted + 1'b1;
		end
	end
	else begin 
		addr_shifted <= address_in[6:3];
	end
end
end

assign addr_shifted_out[i][j] = addr_shifted;
reg [CM_WIDTH-1:0] cm_bit_en_shifted;
logic cm_en_shifted;

//always @(posedge clk) begin
always_comb begin
if (reset)
        cm_bit_en_shifted <= {CM_WIDTH{1'b1}};
else if (chip_en && !start_exec) begin
        if (i==0 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en0;
                cm_en_shifted = cm_en[0];
        end
        else if (i==0 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en1;
                cm_en_shifted = cm_en[1];
        end
        else if (i==0 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en2;
                cm_en_shifted = cm_en[2];
        end
        else if (i==0 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en3;
                cm_en_shifted = cm_en[3];
        end
        else if (i==0 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en4;
                cm_en_shifted = cm_en[4];
        end
        else if (i==0 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en5;
                cm_en_shifted = cm_en[5];
        end
        else if (i==1 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en6;
                cm_en_shifted = cm_en[6];
        end
        else if (i==1 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en7;
                cm_en_shifted = cm_en[7];
        end
        else if (i==1 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en8;
                cm_en_shifted = cm_en[8];
        end
        else if (i==1 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en9;
                cm_en_shifted = cm_en[9];
        end
        else if (i==1 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en10;
                cm_en_shifted = cm_en[10];
        end
        else if (i==1 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en11;
                cm_en_shifted = cm_en[11];
        end
        else if (i==2 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en12;
                cm_en_shifted = cm_en[12];
        end
        else if (i==2 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en13;
                cm_en_shifted = cm_en[13];
        end
        else if (i==2 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en14;
                cm_en_shifted = cm_en[14];
        end
        else if (i==2 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en15;
                cm_en_shifted = cm_en[15];
        end
        else if (i==2 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en16;
                cm_en_shifted = cm_en[16];
        end
        else if (i==2 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en17;
                cm_en_shifted = cm_en[17];
        end
        else if (i==3 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en18;
                cm_en_shifted = cm_en[18];
        end
        else if (i==3 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en19;
                cm_en_shifted = cm_en[19];
        end
        else if (i==3 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en20;
                cm_en_shifted = cm_en[20];
        end
        else if (i==3 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en21;
                cm_en_shifted = cm_en[21];
        end
        else if (i==3 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en22;
                cm_en_shifted = cm_en[22];
        end
else if (i==3 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en23;
                cm_en_shifted = cm_en[23];
        end
        else if (i==4 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en24;
                cm_en_shifted = cm_en[24];
        end
        else if (i==4 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en25;
                cm_en_shifted = cm_en[25];
        end
        else if (i==4 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en26;
                cm_en_shifted = cm_en[26];
        end
        else if (i==4 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en27;
                cm_en_shifted = cm_en[27];
        end
        else if (i==4 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en28;
                cm_en_shifted = cm_en[28];
        end
        else if (i==4 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en29;
                cm_en_shifted = cm_en[29];
        end
        else if (i==5 && j==0)begin
                cm_bit_en_shifted <= cm_bit_en30;
                cm_en_shifted = cm_en[30];
        end
        else if (i==5 && j==1)begin
                cm_bit_en_shifted <= cm_bit_en31;
                cm_en_shifted = cm_en[31];
        end
        else if (i==5 && j==2)begin
                cm_bit_en_shifted <= cm_bit_en32;
                cm_en_shifted = cm_en[32];
        end
        else if (i==5 && j==3)begin
                cm_bit_en_shifted <= cm_bit_en33;
                cm_en_shifted = cm_en[33];
        end
        else if (i==5 && j==4)begin
                cm_bit_en_shifted <= cm_bit_en34;
                cm_en_shifted = cm_en[34];
        end
        else if (i==5 && j==5)begin
                cm_bit_en_shifted <= cm_bit_en35;
                cm_en_shifted = cm_en[35];
        end
else begin
                cm_bit_en_shifted <= {CM_WIDTH{1'b1}};
                cm_en_shifted = 1'b1;
        end
end
else if(start_exec)
        cm_en_shifted = 1'b0;
end


reg [CM_WIDTH-1:0] cm_data_shifted;
//always @(posedge clk) begin
always_comb begin
if (reset) begin
	cm_data_shifted <= {CM_WIDTH{1'b0}};
end
else if (chip_en) begin
	cm_data_shifted <= cm_data;
end
end
assign clk_shifted=~clk;

/*IN22FDX_R1PV_NFVG_W00032B064M02C256 control_mem(
.CLK(clk_shifted),
.CEN(1'b0),
.RDWEN(wr_en_shifted),
.DEEPSLEEP(1'b0),
.POWERGATE(1'b0),
.AW(addr_shifted[3:0]),
.AC(addr_shifted[4]),
.D(cm_data_shifted),
.BW(cm_bit_en_shifted),
.T_LOGIC(1'b0),
.MA_SAWL(1'b0),
.MA_WL(1'b0),
.MA_WRAS(1'b0),
.MA_WRASD(1'b0),
.Q(data_out),
.OBSV_CTL()
);*/

/*always @(posedge clk)
begin
if (r__reset__pff[NUM_RESET_SYNC_STAGES-1])
control_reg_data <= {{CM_WIDTH-21{1'b0}},{21{1'b1}}};
else 
	if (chip_en&&start_exec)
		control_reg_data <= data_out;
end*/

        tile
            tile(
                .clk                                (clk_shifted),
                .reset                              (r__reset__pff[NUM_RESET_SYNC_STAGES-1]),
                .i__flit_in                         (my_flit_in),
                .o__flit_out                        (my_flit_out),
		.control_mem_data_in                (cm_data_shifted),
                .control_mem_en                     (cm_en_shifted),
                .control_mem_bit_en                 (cm_bit_en_shifted),
                .control_mem_addr                   (addr_shifted),
                .control_mem_wr_en                  (wr_en_shifted),
		.is_dm_tile			    (is_dm_tile),
		.start_exec			    (start_exec),
		.loop_start		    	    (loop_start),
		.loop_end		    	    (loop_end),
		.look_up_table		    	    (look_up_table_reg),
		.data_in_dm			    (data_in_dm),
		.data_out_dm			    (data_out_dm),
		.addr_dm			    (addr_dm),
		.bit_en				    (bit_en),
		.rd_en_dm			    (rd_en_dm),
		.wr_en_dm			    (wr_en_dm)
            );
    end
end

for(i = 0; i < NUM_TILES_Y; i++)
begin: y_connection
    for(j = 0; j < NUM_TILES_X; j++)
    begin: x_connection

        // East
        if(j < (NUM_TILES_X-2))
        begin: flit_east
            assign w__flit_in[i][j][EAST]   = w__flit_out[i][j + 2][WEST];
        end
        else if(j == (NUM_TILES_X-2))
        begin
            assign w__flit_in[i][j][EAST]   = w__flit_in[i][j + 1][EAST];
        end
	else
	begin
            assign w__flit_in[i][j][EAST]   = w__flit_in[i][j - 1][EAST];
        end

        // South
        if(i >= 2)
        begin: flit_south
            assign w__flit_in[i][j][NORTH]      = w__flit_out[i - 2][j][SOUTH];
        end
	else if(i >= 1)
	begin 
            assign w__flit_in[i][j][NORTH]      = w__flit_out[i - 1][j][NORTH];
        end
        else
        begin 
            assign w__flit_in[i][j][NORTH]      = w__flit_out[i + 1][j][NORTH];
        end

        // West
        if(j >= 2)
        begin: flit_west
            assign w__flit_in[i][j][WEST]   = w__flit_out[i][j - 2][EAST];
        end
	else if(j >= 1)
	begin 
            assign w__flit_in[i][j][WEST]   = w__flit_out[i][j - 1][WEST];
        end
        else
        begin 
            assign w__flit_in[i][j][WEST]   = w__flit_out[i][j + 1][WEST];
        end

        // North
        if(i < (NUM_TILES_Y-2))
        begin: flit_north
            assign w__flit_in[i][j][SOUTH]      = w__flit_out[i + 2][j][NORTH];
        end
	else if(i == (NUM_TILES_Y-2))
	begin 
            assign w__flit_in[i][j][SOUTH]      = w__flit_out[i + 1][j][SOUTH];
        end
        else
        begin 
            assign w__flit_in[i][j][SOUTH]      = w__flit_out[i - 1][j][SOUTH];
        end
   

        //----------------------------------------------------------------------

    end
end

endgenerate

/*

assign addr_dm_shifted0 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[0][0];
assign bit_en_shifted0 = (!start_exec) ? dm_bit_en : bit_en_out[0][0];
assign data_in_dm_shifted0 = (!start_exec) ? dm_data : data_in_dm_out[0][0];
assign chip_en_dm_shifted0 = (!start_exec) ? chip_en_dm0 : rd_en_dm_out[0][0];
assign wr_en_dm_shifted0 = (!start_exec) ? read_write : wr_en_dm_out[0][0];

assign addr_dm_shifted8 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[1][0];
assign bit_en_shifted8 = (!start_exec) ? dm_bit_en : bit_en_out[1][0];
assign data_in_dm_shifted8 = (!start_exec) ? dm_data : data_in_dm_out[1][0];
assign chip_en_dm_shifted8 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[1][0];
assign wr_en_dm_shifted8 = (!start_exec) ? read_write : wr_en_dm_out[1][0];

assign addr_dm_shifted16 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[2][0];
assign bit_en_shifted16 = (!start_exec) ? dm_bit_en : bit_en_out[2][0];
assign data_in_dm_shifted16 = (!start_exec) ? dm_data : data_in_dm_out[2][0];
assign chip_en_dm_shifted16 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[2][0];
assign wr_en_dm_shifted16 = (!start_exec) ? read_write : wr_en_dm_out[2][0];

assign addr_dm_shifted24 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[3][0];
assign bit_en_shifted24 = (!start_exec) ? dm_bit_en : bit_en_out[3][0];
assign data_in_dm_shifted24 = (!start_exec) ? dm_data : data_in_dm_out[3][0];
assign chip_en_dm_shifted24 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[3][0];
assign wr_en_dm_shifted24 = (!start_exec) ? read_write : wr_en_dm_out[3][0];

assign addr_dm_shifted32 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[4][0];
assign bit_en_shifted32 = (!start_exec) ? dm_bit_en : bit_en_out[4][0];
assign data_in_dm_shifted32 = (!start_exec) ? dm_data : data_in_dm_out[4][0];
assign chip_en_dm_shifted32 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[4][0];
assign wr_en_dm_shifted32 = (!start_exec) ? read_write : wr_en_dm_out[4][0];

assign addr_dm_shifted40 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[5][0];
assign bit_en_shifted40 = (!start_exec) ? dm_bit_en : bit_en_out[5][0];
assign data_in_dm_shifted40 = (!start_exec) ? dm_data : data_in_dm_out[5][0];
assign chip_en_dm_shifted40 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[5][0];
assign wr_en_dm_shifted40 = (!start_exec) ? read_write : wr_en_dm_out[5][0];

assign addr_dm_shifted48 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[0][5];
assign bit_en_shifted48 = (!start_exec) ? dm_bit_en : bit_en_out[0][5];
assign data_in_dm_shifted48 = (!start_exec) ? dm_data : data_in_dm_out[0][5];
assign chip_en_dm_shifted48 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[0][5];
assign wr_en_dm_shifted48 = (!start_exec) ? read_write : wr_en_dm_out[0][5];

assign addr_dm_shifted56 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[1][5];
assign bit_en_shifted56 = (!start_exec) ? dm_bit_en : bit_en_out[1][5];
assign data_in_dm_shifted56 = (!start_exec) ? dm_data : data_in_dm_out[1][5];
assign chip_en_dm_shifted56 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[1][5];
assign wr_en_dm_shifted56 = (!start_exec) ? read_write : wr_en_dm_out[1][5];

assign addr_dm_shifted7 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[2][5];
assign bit_en_shifted7 = (!start_exec) ? dm_bit_en : bit_en_out[2][5];
assign data_in_dm_shifted7 = (!start_exec) ? dm_data : data_in_dm_out[2][5];
assign chip_en_dm_shifted7 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[2][5];
assign wr_en_dm_shifted7 = (!start_exec) ? read_write : wr_en_dm_out[2][5];

assign addr_dm_shifted15 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[3][5];
assign bit_en_shifted15 = (!start_exec) ? dm_bit_en : bit_en_out[3][5];
assign data_in_dm_shifted15 = (!start_exec) ? dm_data : data_in_dm_out[3][5];
assign chip_en_dm_shifted15 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[3][5];
assign wr_en_dm_shifted15 = (!start_exec) ? read_write : wr_en_dm_out[3][5];

assign addr_dm_shifted23 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[4][5];
assign bit_en_shifted23 = (!start_exec) ? dm_bit_en : bit_en_out[4][5];
assign data_in_dm_shifted23 = (!start_exec) ? dm_data : data_in_dm_out[4][5];
assign chip_en_dm_shifted23 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[4][5];
assign wr_en_dm_shifted23 = (!start_exec) ? read_write : wr_en_dm_out[4][5];

assign addr_dm_shifted31 = (!start_exec) ? address_in[DM_DEPTH_BITS:1] : addr_dm_out[5][5];
assign bit_en_shifted31 = (!start_exec) ? dm_bit_en : bit_en_out[5][5];
assign data_in_dm_shifted31 = (!start_exec) ? dm_data : data_in_dm_out[5][5];
assign chip_en_dm_shifted31 = (!start_exec) ? chip_en_dm8 : rd_en_dm_out[5][5];
assign wr_en_dm_shifted31 = (!start_exec) ? read_write : wr_en_dm_out[5][5];


wire [CM_WIDTH-1:0] cm_data_out;
reg [15:0] cm_addr_out;

always @(posedge clk) begin
if (reset) begin
        cm_addr_out <= {16{1'b0}};
end
else if (chip_en&&read_write) begin
	cm_addr_out <= address_in [15:0];
end
end



always @(posedge clk) begin
if (reset) begin
        data_out <= {DATA_WIDTH{1'b0}};
	data_out_valid <= 1'b0;
end
else if (chip_en) begin
	if (read_write && data_addr_valid_reg==2'b11 && data_or_addr_reg==1'b0) begin
        if(address_in[18]==1'b1 && address_in[19]==1'b0) begin
			data_out_valid <= 1'b1;
			data_out <= dm_data_out;
        end
	//else begin
        else if(cm_addr_out[13]==1'b0 && cm_addr_out[14]==1'b0) begin
			data_out_valid <= 1'b1;
		if(cm_addr_out[2:1]==2'b00) begin
			data_out <= cm_data_out[15:0];
		end
		else if(cm_addr_out[2:1]==2'b01) begin
			data_out <= cm_data_out[31:16];
		end
		else if(cm_addr_out[2:1]==2'b10) begin
			data_out <= cm_data_out[47:32];
		end
		else if(cm_addr_out[2:1]==2'b11) begin
			data_out <= cm_data_out[63:48];
		end
        end
	else begin
		data_out <= {DATA_WIDTH{1'b0}};
		data_out_valid <= 1'b0;
	end
	end
	else begin
		data_out <= {DATA_WIDTH{1'b0}};
		data_out_valid <= 1'b0;
	end
end
end
*/

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Reset synchronizer
//------------------------------------------------------------------------------
always_ff @ (posedge clk)
begin
    if(reset == 1'b1)
    begin
        r__reset__pff <= '1;
    end
    else if (chip_en) 
    begin
        r__reset__pff <= {r__reset__pff[NUM_RESET_SYNC_STAGES-2:0], 1'b0};
    end
end
//------------------------------------------------------------------------------


endmodule

